local Blizzard = SpeedyActions:NewModule("Blizzard", "AceEvent-3.0")
local _G = getfenv(0)

local buttonsScanned, bindingByKey = {}, {}
local overrideFrame = CreateFrame("Frame")
local overrideList = {
	["ActionButton%d"] = "scan", ["MultiBarBottomLeftButton%d"] = "scan", ["MultiBarBottomRightButton%d"] = "scan",
	["MultiBarLeftButton%d"] = "scan", ["MultiBarRightButton%d"] = "scan", ["MultiCastActionButton%d"] = "scan", ["MultiCastRecallSpellButton"] = "single",
	["VehicleMenuBarActionButton%d"] = "scan", ["ShapeshiftButton%d"] = "scan", ["PetActionButton%d"] = "scan", ["BonusActionButton%d"] = "scan",
}

function Blizzard:OnInitialize()
	self:RegisterEvent("UPDATE_BINDINGS")

	self:FindButtons()
	self:UPDATE_BINDINGS()
end

function Blizzard:FindButtons()
	for buttonTemplate, type in pairs(overrideList) do
		if( type == "single" ) then
			local button = _G[buttonTemplate]
			buttonsScanned[string.upper(button:GetName())] = button
		else
			local id = 1
			while( true ) do
				local button = _G[string.format(buttonTemplate, id)]
				if( not button ) then break end
				buttonsScanned[string.upper(button:GetName())] = button
				id = id + 1
			end
		end
	end
end

function Blizzard:UPDATE_BINDINGS()
	table.wipe(bindingByKey)
	
	for i=1, GetNumBindings() do
		local action, bindingOne, bindingTwo = GetBinding(i)
		if( bindingOne ) then bindingByKey[bindingOne] = GetBindingByKey(bindingOne) end
		if( bindingTwo ) then bindingByKey[bindingTwo] = GetBindingByKey(bindingTwo) end
	end
	
	-- Reoverride button again to apply the new bindings
	if( InCombatLockdown() ) then
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "RebindScanned")
		return
	end
	
	self:RebindScanned()
end

function Blizzard:RebindScanned()
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	SpeedyActions:ReleaseButtons("blizzard")
	ClearOverrideBindings(overrideFrame)
	
	for key, action in pairs(bindingByKey) do
		-- Bindings set through SetBindingClick
		if( string.match(action, "^CLICK") ) then
			local frame, mouseButton = string.match(action, "^CLICK (.+):(.+)")
			local button = frame and buttonsScanned[string.upper(frame)]
			if( button ) then
				self:RebindButton(button, key, mouseButton)
			end
		
		-- Set through SetBindingSpell
		elseif( string.match(action, "^SPELL") ) then
			local button = SpeedyActions:GetButton("blizzard")
		
			button:SetAttribute("spell", string.match(action, "^SPELL (.+)"))
			self:RebindButton(button, key)
		
		-- Set through SetBindingItem
		elseif( string.match(action, "^ITEM") ) then
			local button = SpeedyActions:GetButton("blizzard")
		
			button:SetAttribute("item", string.match(action, "^ITEM (.+)"))
			self:RebindButton(button, key)
		
		-- Set through SetBindingMacro				
		elseif( string.match(action, "^MACRO") ) then
			local button = SpeedyActions:GetButton("blizzard")
			
			button:SetAttribute("macro", string.match(action, "^MACRO (.+)"))
			self:RebindButton(button, key)
			
		-- Set through SetBinding to a multi cast command
		elseif( string.match(action, "MULTICASTSUMMONBUTTON(%d+)") ) then
			local id = string.match(action, "MULTICASTSUMMONBUTTON(%d+)")
			local button = SpeedyActions:GetButton("blizzard")

			button:SetAttribute("spell", TOTEM_MULTI_CAST_SUMMON_SPELLS[tonumber(id)])
			self:RebindButton(button, key)

		-- Set through SetBinding to a recall command
		elseif( string.match(action, "MULTICASTRECALLBUTTON(%d+)") ) then
			local id = string.match(action, "MULTICASTRECALLBUTTON(%d+)")
			local button = SpeedyActions:GetButton("blizzard")

			button:SetAttribute("spell", TOTEM_MULTI_CAST_RECALL_SPELLS[tonumber(id)])
			self:RebindButton(button, key)
			
		-- Generic bindings set through a command in SetBinding
		else
			local button = buttonsScanned[self:ConvertBindAction(action)]
			if( button ) then
				self:RebindButton(button, key)
			end
		end
	end
end

-- Converts the multi bars which do not use the "standard" naming
function Blizzard:ConvertBindAction(action)
	action = string.gsub(action, "MULTIACTIONBAR4BUTTON", "MultiBarLeftButton")
	action = string.gsub(action, "MULTIACTIONBAR3BUTTON", "MultiBarRightButton")
	action = string.gsub(action, "MULTIACTIONBAR2BUTTON", "MultiBarBottomRightButton")
	action = string.gsub(action, "MULTIACTIONBAR1BUTTON", "MultiBarBottomLeftButton")

	return string.upper(action)
end

function Blizzard:RebindButton(button, key, mouseButton)
	button:RegisterForClicks("AnyDown")
	SetOverrideBindingClick(overrideFrame, true, key, button:GetName(), mouseButton)
end



